/**
 * An application object to inject a bouncy castle into Amanda's Play
 * World
 *
 * This Class is used extensively in bt.sie.msss.Play.AOShastl and so
 * any changes here need to be reflected there.
 *
 * @author Tim Regan
 * @version 0.1 30 Nov 96
 */
import sony.vsao.aoscript;

public class AOCastle extends aoscript
{

  /**
   * The name of the application object
   */
  public String myName = "Bouncy Castle";
  /**
   * The file containing the application objects VRML
   */
  public String myVRML = "ao/castle.wrl";
  /**
   * The application objects index : -1 if not yet set
   */
  public int sensorIndex = -1;
  /**
   * The maximum number of avatars bouncing after which the teleport
   * sensors will be enabled
   */
  public static Integer maximumBouncing = new Integer(5);
  /**
   * The number of avatars bouncing.
   */
  public Integer bouncingAvatars;
  /**
   * The last time I was asked to add a bouncer
   */
  public long lastBouncerEnteredTime = 0;

  /**
   * Called by the AO Manager when the object is loaded
   */
  public void aoInit()
  {
    SetServerName("mars.portal.bt.co.uk");
    SetPort(6186);
    //SetServerName("localhost");
    //SetPort(5126);
    bouncingAvatars = new Integer(0);
    setUpStart();
  }

  /**
   * Set up and position my VRML objects. Return 1 for success and 0 for failure
   */
  public long setUpStart()
  {
    long result = 1;

    sensorIndex =  CreateObject(0,myVRML,myName,AO_COF_SYSTEM);
    if ( sensorIndex >= 0 )
      {
	result *= SetPosition(sensorIndex,-50,1,-50);
	result *= UpdatePosition(sensorIndex);
      }
    else
      {
	result = 0;
      }
    return result;
  }

  /**
   * Called when an avatar gets on the bouncy castle
   */
  public void bouncerEntered(int iIndex, int senderID, String arg)
  {
    // I've added the following check because I suspect that if
    // peoples browsers crash, they exit, or they are disconnected
    // from the bureau then the count of who is bouncing is
    // wrong. Hence we'll set it back to zero if no-one has entered
    // for 10 minutes or 600000 milliseconds
    if ( (System.currentTimeMillis() - lastBouncerEnteredTime) > 600000 )
	 {
	   bouncingAvatars = new Integer(1);
	 }
    else
      {
	bouncingAvatars = new Integer(bouncingAvatars.intValue() + 1);
      }
    println("bouncerEntered called on AO bringing total bouncing to " + bouncingAvatars);
    if ( bouncingAvatars.equals(maximumBouncing) )
      {
	//println("AO asking clients to enable teleporters");
	//SendEvent(sensorIndex,"enableTeleporter","");
      }
    lastBouncerEnteredTime = System.currentTimeMillis();
    SendEvent(sensorIndex,"updateBouncerNumber",bouncingAvatars.toString());
  }

  /**
   * Called when an avatar gets off the bouncy castle
   */
  public void bouncerExited(int iIndex, int senderID, String arg)
  {
    bouncingAvatars = new Integer(bouncingAvatars.intValue() - 1);
    println("bouncerExited called on AO bringing total bouncing to " + bouncingAvatars);
    SendEvent(sensorIndex,"updateBouncerNumber",bouncingAvatars.toString());
  }

  /**
   * Called when an avatar bounces so high they are teleported to
   * another world
   */
  public void teleporterEntered(int iIndex, int senderID, String arg)
  {
    println("Christ knows what to do here");
  }

  /**
   * I have to implement something like this locally so that we get
   * useful messages in the log when running the AO as a service.
   */
  protected void println(String toPrint)
  {
    System.err.println(toPrint);
    PutLogMessage(toPrint + "\n");
  }

}
