/**
 *  An application object to manage the VRML shuffle board
 *
 *  This Class is used extensively in bt.sie.msss.Play.AOShastl and so
 *  any changes here need to be reflected there.
 *
 *  @author Tim Regan
 *  @version 0.1 30 Nov 96
 */
import sony.vsao.aoscript;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.EOFException;


public class AOShuffleBoard extends aoscript
{

  /**
   * The name of the application object
   */
  public String myName = "Shuffle Board";
  /**
   * The file containing the application objects VRML
   */
  public String myVRML = "ao/shuffle.wrl";
  /**
   * The application objects index : -1 if not yet set
   */
  public int sensorIndex = -1;
  /**
   * The scores file
   */
  //public String scoresFile = "/ProgramFiles/Sony/AO-Manager/java/usrclass/play/files/scores.txt";
  public String scoresFile = "/Program Files/Sony/Community Place AO/usrclass/play/files/scores.txt";
  /**
   * Maximum length of the scores file
   */
  public int maxScoreFileBytes = 1000;
  /**
   * The top ten scores
   */
  public Integer scores[];
  /**
   * The top ten scorers
   */
  public String scorers[];


  /**
   * Called by the AO Manager when the object is loaded
   */
  public void aoInit() throws IOException
  {
    scores = new Integer[10];
    scorers = new String[10];
    SetServerName("mars.portal.bt.co.uk");
    SetPort(6186);
    //SetServerName("localhost");
    //SetPort(5126);
    setUpStart();
    readScores();
  }

  /**
   * Set up and position my VRML objects. Return 1 for success and 0
   * for failure
   */
  public long setUpStart()
  {
    long result = 1;
    float matrix[][] = {{ -1, 0, 0 },
			{ 0, 1, 0 },
			{ 0, 0, -1 }};

    sensorIndex =  CreateObject(50,myVRML,myName,AO_COF_SYSTEM); // 50 is Aura size
    if ( sensorIndex >= 0 )
      {
	//result *= SetPosition(sensorIndex,-45,13,70);
	result *= SetPosition(sensorIndex,-36,6,70);
	result *= SetRotate(sensorIndex,matrix);
	result *= UpdatePosition(sensorIndex);
      }
    else
      {
	result = 0;
      }
    return result;
  }

  /**
   * Read the scores in
   */
  public void readScores() throws IOException
  {
    DataInputStream scoreStream;
    String scoreString;
    StringTokenizer scoreStringTokens;
    byte scoreBytes[] = new byte[maxScoreFileBytes];

    scoreStream = new DataInputStream(new FileInputStream(scoresFile));
    try
      {
	// I wanted to write scoreStream.readFully(scoreBytes) but it
	// throws a EOFException! I also do not understand why
	// scoreStream.available() answers a larger number than the
	// number of characters in the score file. Hence...
	for ( int i = 0 ; i < maxScoreFileBytes ; i++ )
	  {
	    scoreBytes[i] = scoreStream.readByte();
	  }
      }
    catch (EOFException e) {}
    catch (IOException e)
      {
      	println("Exception whilst reading bytes : " + e);
	e.printStackTrace();
	return;
      }
    scoreString = new String(scoreBytes,0);
    scoreStringTokens = new StringTokenizer(scoreString,"\n\r");
    for ( int i = 0 ; scoreStringTokens.hasMoreTokens() ; i++ )
      {
	if ( (i%2)==0 )
	  {
	    scores[i/2] = new Integer(scoreStringTokens.nextToken());
	    println("Adding score " + scores[i/2] + " at index " + (i/2));
	  }
	else
	  {
	    scorers[i/2] = scoreStringTokens.nextToken();
	    if ( scorers[i/2].length() > 7 )
	      {
		scorers[i/2] = scorers[i/2].substring(0,7);
	      }
	    println("Adding scorer " + scorers[i/2] + " at index " + (i/2));
	  }
      }
    scoreStream.close();
  }

  /**
   * Called by clients on start up
   */
  public void whatAreTheScores(int iIndex, int senderID, String arg) throws IOException
  {
    println("Asked for score by " + senderID);
    writeNewScoreToClient(senderID);
  }

  /**
   * Called by clients when they have finnished
   */
  public void playFinnished(int iIndex, int senderID, String arg) throws IOException
  {
    Integer score = new Integer(arg);
    String scorer =
      InqObjectNameInMyAura(sensorIndex,senderID);
    if ( scorer.length() > 7 ) scorer = scorer.substring(0,7);
    for ( int i = 0 ; i < 10 ; i++ )
      {
	if ( score.intValue() <= scores[i].intValue() )
	  {
	    for ( int j = 9 ; j > i ; j-- )
	      {
		scores[j] = scores[j-1];
		scorers[j] = scorers[j-1];
	      }
	    scores[i] = score;
	    println("Adding score " + score + " to roll of honour");
	    scorers[i] = scorer;
	    writeNewScore();
	    break;
	  }
      }
  }

  /**
   * Write the new scores to the file and all the clients
   */
  public void writeNewScore() throws IOException
  {
    DataOutputStream scoreStream;
    StringBuffer scoreString = new StringBuffer();

    scoreStream = new DataOutputStream(new FileOutputStream(scoresFile));
    for ( int i = 0 ; i < 10 ; i++ )
      {
	if ( scores[i] != null )
	  {
	    scoreString = scoreString.append(scores[i].intValue());
	    scoreString = scoreString.append('\n');
	    scoreString = scoreString.append(scorers[i]);
	    scoreString = scoreString.append('\n');
	  }
      }
    scoreStream.writeBytes(scoreString.toString());
    scoreStream.flush();
    scoreStream.close();
    SendEvent(sensorIndex,"updateScores",scoreString.toString());
  }

  /**
   * Write the scores to a client
   */
  public void writeNewScoreToClient(int clientId) throws IOException
  {
    StringBuffer scoreString = new StringBuffer();
    long sendSuccess = 0;

    for ( int i = 0 ; i < 10 ; i++ )
      {
	if ( scores[i] != null )
	  {
	    scoreString = scoreString.append(scores[i].intValue());
	    scoreString = scoreString.append('\n');
	    scoreString = scoreString.append(scorers[i]);
	    scoreString = scoreString.append('\n');
	  }
      }
    sendSuccess = SendppEvent(sensorIndex,clientId,"updateScores",scoreString.toString());
    println("Sent scores to " + clientId + " with result " + sendSuccess);
  }

  /**
   * I have to implement something like this locally so that we get
   * useful messages in the log when running the AO as a service.
   */
  protected void println(String toPrint)
  {
    System.err.println(toPrint);
    PutLogMessage(toPrint + "\n");
  }

}
