/**
 * An application object to integrate the bouncy castle and the
 * shuffle board application objects
 *
 * @author Tim Regan
 * @version 0.1 20 Nov 96
 */
import sony.vsao.aoscript;
import AOCastle;
import AOShuffleBoard;
import java.io.IOException;

public class Play extends aoscript
{

  /**
   * The castle AO
   */
  protected AOCastle myCastle;
  /**
   * The shuffle board AO
   */
  protected AOShuffleBoard myShuffleBoard;

  /**
   * The constructor to make the two AO's : shuffle boar and castle
   */
  public Play()
  {
    myCastle = new AOCastle();
    myShuffleBoard = new AOShuffleBoard();
  }

  /**
   * Called by the AO Manager when the object is loaded
   */
  public void aoInit()
  {
    // Generic stuff :
    //Bureau configuration = new Bureau("/ProgramFiles/Sony/AO-Manager/java/usrclass/play/files/play.conf");
    Bureau configuration = new Bureau("/Program Files/Sony/Community Place AO/usrclass/play/files/play.conf");
    SetServerName(configuration.getBureauName());
    SetPort(configuration.getBureauPort());
    SetRetry(300);

    // Bouncy castle stuff :
    myCastle.bouncingAvatars = new Integer(0);
    myCastle.setUpStart();
    // Shuffle board stuff :
    myShuffleBoard.scores = new Integer[10];
    myShuffleBoard.scorers = new String[10];
    myShuffleBoard.setUpStart();
    try
      {
	myShuffleBoard.readScores();
      }
    catch (IOException e)
      {
      	println("Exception whilst reading scores : " + e);
	e.printStackTrace();
      }
  }

  /**
   * Called when an avatar gets on the bouncy castle
   */
  public void bouncerEntered(int iIndex, int senderID, String arg)
  {
    myCastle.bouncerEntered(iIndex,senderID,arg);
  }

  /**
   * Called when an avatar gets off the bouncy castle
   */
  public void bouncerExited(int iIndex, int senderID, String arg)
  {
    myCastle.bouncerExited(iIndex,senderID,arg);
  }

  /**
   * Called when an avatar bounces so high they are teleported to
   * another world
   */
  public void teleporterEntered(int iIndex, int senderID, String arg)
  {
    myCastle.teleporterEntered(iIndex,senderID,arg);
  }

  /**
   * Called by clients on start up
   */
  public void whatAreTheScores(int iIndex, int senderID, String arg) throws IOException
  {
    myShuffleBoard.whatAreTheScores(iIndex,senderID,arg);
  }

  /**
   * Called by clients when they have finnished
   */
  public void playFinnished(int iIndex, int senderID, String arg) throws IOException
  {
    myShuffleBoard.playFinnished(iIndex,senderID,arg);
  }

  /**
   * I have to implement something like this locally so that we get
   * useful messages in the log when running the AO as a service.
   */
  protected void println(String toPrint)
  {
    System.err.println(toPrint);
    PutLogMessage(toPrint + "\n");
  }

}
